@@ -80,12 +80,15 @@ available.views <- CRAN.views <- function(repos = NULL, ...)
   return(rval)
 }
 
-install.views <- function(views, coreOnly = FALSE, repos = NULL, ...)
+.get_pkgs_from_ctv_or_repos <- function(views, coreOnly = FALSE, repos = NULL)
 {
+  ## views is already a "ctv"
   if(inherits(views, "ctv")) {
     views <- list(views)
     class(views) <- "ctvlist"
   }
+  
+  ## views is a string that needs to be turned to "ctv"
   if(!inherits(views, "ctvlist")) {
     ## get CRAN views and extract names of available views
     cranviews <- available.views(repos = repos)
@@ -100,93 +103,74 @@ install.views <- function(views, coreOnly = FALSE, repos = NULL, ...)
     views <- cranviews[as.vector(unlist(whichviews))]
     class(views) <- "ctvlist"
   }
-  
-  ## install packages for each view
+
+  ## compute list of unique packages for each repository
   coreOnly <- rep(coreOnly, length.out = length(views)) 
-   
-  for(i in seq(along = views)) {
-    pkgs <- if(coreOnly[i]) views[[i]]$packagelist$name[views[[i]]$packagelist$core]
-      else views[[i]]$packagelist$name
-    install.packages(pkgs, repos = views[[i]]$repository, ...)
-  }
+  pkgs <- lapply(seq(along = views), function(i) {
+    pkgs <- if(!coreOnly[i]) views[[i]]$packagelist$name
+      else views[[i]]$packagelist$name[views[[i]]$packagelist$core]
+    cbind(pkgs, views[[i]]$repository)
+  })
+  pkgs <- do.call("rbind", pkgs)
+  pkgs <- tapply(pkgs[,1], pkgs[,2], function(x) sort(unique(x)))
   
+  return(pkgs)
+}
+
+install.views <- function(views, coreOnly = FALSE, repos = NULL, ...)
+{
+  pkgs <- .get_pkgs_from_ctv_or_repos(views = views, coreOnly = coreOnly, repos = repos)
+  for(i in seq(along = pkgs)) install.packages(pkgs[[i]], repos = names(pkgs)[i], ...)    
+  invisible()
+}
+
+download.views <- function(views, destdir, coreOnly = FALSE, repos = NULL, ...)
+{
+  pkgs <- .get_pkgs_from_ctv_or_repos(views = views, coreOnly = coreOnly, repos = repos)
+  for(i in seq(along = pkgs)) download.packages(pkgs[[i]], destdir = destdir, repos = names(pkgs)[i], ...)    
   invisible()
 }
 
 update.views <- function(views, coreOnly = FALSE, repos = NULL, lib.loc = NULL, ...)
 {
-  if(inherits(views, "ctv")) {
-    views <- list(views)
-    class(views) <- "ctvlist"
-  }
-  if(!inherits(views, "ctvlist")) {
-    ## get CRAN views and extract names of available views
-    cranviews <- available.views(repos = repos)
-    availnames <- sapply(seq(along = cranviews), function(i) cranviews[[i]]$name)
-
-    whichviews <- lapply(views, function(z) {
-      rval <- which(z == availnames)
-      if(length(rval) > 0) rval[1] else numeric(0)
-    })
-    unavail <- which(sapply(whichviews, length) < 1)
-    if(length(unavail) > 0) warning(paste("CRAN task view", views[unavail], "not available", collapse = "\n"))
-    views <- cranviews[as.vector(unlist(whichviews))]
-    class(views) <- "ctvlist"
-  }
-  
-  ## get full package list
-  coreOnly <- rep(coreOnly, length.out = length(views)) 
-  pkgs <- lapply(seq(along = views), function(i)
-    if(coreOnly[i]) views[[i]]$packagelist$name[views[[i]]$packagelist$core]
-      else views[[i]]$packagelist$name)
-  pkgs <- sort(unique(unlist(pkgs)))
-
-  ## getOption("repos")
-  if(is.null(repos)) repos <- ifelse(is.null(getOption("repos")), getOption("CRAN"), getOption("repos"))
-  if("@CRAN@" %in% repos && interactive()) {
-      cat(gettext("--- Please select a CRAN mirror for use in this session ---\n"))
-      flush.console()
-      chooseCRANmirror()
-      m <- match("@CRAN@", repos)
-      nm <- names(repos)
-      repos[m] <- getOption("repos")["CRAN"]
-      if(is.null(nm)) nm <- rep("", length(repos))
-      nm[m] <- "CRAN"
-      names(repos) <- nm
-  }
-  if("@CRAN@" %in% repos) stop("trying to use CRAN without setting a mirror")
-  
-  ## query available packages
-  apkgs <- available.packages(contriburl = contrib.url(repos))
-
-  ## compute intersection
-  unavail <- which(!(pkgs %in% as.character(apkgs[,1])))
-  if(length(unavail) > 0) {
-    warning(sprintf("The following packages are not available: %s", paste(pkgs[unavail], collapse = ", ")))
-    pkgs <- pkgs[-unavail]
-  }
-  apkgs <- apkgs[pkgs,,drop=FALSE]
-
-  ## query installed packages
-  ipkgs <- installed.packages(lib.loc = lib.loc)
-  ipkgs <- ipkgs[which(ipkgs[,"Package"] %in% pkgs),,drop=FALSE]
-  
-  ## determine which packages need to be updated
-  if(NROW(ipkgs) > 0) {    
-    not_installed <- which(!(pkgs %in% ipkgs[,1]))
+  pkg_list <- .get_pkgs_from_ctv_or_repos(views = views, coreOnly = coreOnly, repos = repos)
     
-    cpkgs <- if(length(not_installed) > 0) pkgs[-not_installed] else pkgs
-    get_highest_version <- function(x)
-      as.character(max(package_version(ipkgs[which(ipkgs[,1] %in% x),"Version"])))
-    not_uptodate <- which(package_version(apkgs[cpkgs, "Version"]) >
-      package_version(sapply(cpkgs, get_highest_version)))
-    pkgs <- sort(c(pkgs[not_installed], cpkgs[not_uptodate]))
-  }
+  for(i in seq(along = pkg_list)) {
+    ## package and repos name(s)
+    pkgs <- pkg_list[[i]]
+    repos <- names(pkg_list)[i]
+        
+    ## query available packages
+    apkgs <- available.packages(contriburl = contrib.url(repos))
 
-  ## install packages required
-  apkgs <- apkgs[pkgs,,drop=FALSE]
-  if(NROW(apkgs) > 0) install.packages(apkgs[,1], contriburl = apkgs[,"Repository"],
-    lib = lib.loc, ...)
+    ## compute intersection
+    unavail <- which(!(pkgs %in% as.character(apkgs[,1])))
+    if(length(unavail) > 0) {
+      warning(sprintf("The following packages are not available: %s", paste(pkgs[unavail], collapse = ", ")))
+      pkgs <- pkgs[-unavail]
+    }
+    apkgs <- apkgs[pkgs,,drop=FALSE]
+
+    ## query installed packages
+    ipkgs <- installed.packages(lib.loc = lib.loc)
+    ipkgs <- ipkgs[which(ipkgs[,"Package"] %in% pkgs),,drop=FALSE]
+  
+    ## determine which packages need to be updated
+    if(NROW(ipkgs) > 0) {    
+      not_installed <- which(!(pkgs %in% ipkgs[,1]))
+    
+      cpkgs <- if(length(not_installed) > 0) pkgs[-not_installed] else pkgs
+      get_highest_version <- function(x)
+        as.character(max(package_version(ipkgs[which(ipkgs[,1] %in% x),"Version"])))
+      not_uptodate <- which(package_version(apkgs[cpkgs, "Version"]) >
+        package_version(sapply(cpkgs, get_highest_version)))
+      pkgs <- sort(c(pkgs[not_installed], cpkgs[not_uptodate]))
+    }
+
+    ## install packages required
+    apkgs <- apkgs[pkgs,,drop=FALSE]
+    if(NROW(apkgs) > 0) install.packages(apkgs[,1], contriburl = apkgs[,"Repository"], lib = lib.loc, ...)
+  }
   
   invisible()
 }
